function result = errmesg(fieldname, fieldtype)
% ERRMESG(fieldname, fieldtype)
%
% Print an appropriate error message for the field named fieldname
% of type fieldtype, where fieldtype is one of:
%  'fun' -- function.
%  'num' -- number.
%
% Use it something like this (where f_of_x is a field uicontrol, and where
% x has been defined to something useful):
%
%    error_happend = 0;
%    f_of_x_string = vectorize(cleanup(get(f_of_x, 'string')));
%    eval([f_of_x_string ';'], 'error_happend = 1;');
%    if error_happened
%        errmesg('f(x)', 'fun');
%        error(lasterr);
%    end
%    f = eval(f_of_x_string);
%
% Or, use it with TRY...CATCH...END.
%
% HISTORY:
% Shorter messages -- tinsel, 8/98
% Created by Tom Insel, January 1998.
% Error text adapted from edu_err.m by Julie Mitchell, 1995.
%
% NEEDS: MATLAB 5

if nargin < 1, fieldname = '???'; end
if nargin < 2, fieldtype = '???'; end

if strcmp(fieldtype, 'fun')
	longtype = 'function';
elseif strcmp(fieldtype, 'num')
	longtype = 'number';
else
	longtype = 'value';
end

d = errordlg({...
	['There is a problem evaluating the ' longtype ' in the'],...
	['field ' fieldname '.  Check you have entered it correctly.']},...
	'Input Error');

% uiwait(d);	
