function result = iterint(cmd)
% DOUBLE INTEGRALS
%
% A module for setting up double integrals and visualizing the area
% of integration.
%
% Enter the limits of integration into the blue and pink boxes
% and make sure that you have chosen the correct order of integration
% from the pop-up menu.  If nothing appears in the right part of
% the window, you've set the integral up wrong.
%
% 2.1/19-July-1998


% REQUIRES: cleanup.m, comet.m
%
% HISTORY:
% - Original by Alex Gottlieb (Summer 1996)
% - Revised by Tom Insel (February 1998)
% - MATLAB 5.2 version by Tom Insel (June 1998)
%
% TODO/BUGS:
% - error detection.
%   check for open terms, in limits/differential.
% - speed up the estimated integral -- I don't exactly understand the algorithm
% - the layout is still not that great.
%
% TESTED: MATLAB 4.2c.1 for Macintosh

persistent lo_outer hi_outer lo_inner hi_inner integrand differential_menu integral_label

math_bkgnd = [0.6 0.8 1.0];
math_field = [1 1 1];
math_inner = [1 .6 .6];
math_outer = [.6 .6 1];

if ~nargin,

	control_figures = findobj('tag', 'iterint_main');
	
	if length(control_figures) == 0
		cmd = 'new';
	else
		figure(control_figures(1));
		cmd = 'NOCOMMAND';
	end

end

if strcmp(cmd,'new')

	fig_han = figure(...
		'name', 'Double Integrals',...
		'Tag', 'iterint_main',...
		'numbertitle', 'off',...
		'position', [10 10 600 400],...
		'color', 'w',...
		'renderer', 'zbuffer',...
		'resize', 'off');

	uicontrol(fig_han,...
		'style', 'frame',...
		'backgroundcolor', math_bkgnd,...
		'position', [-5 -5 205 410]);

	%
	
	uicontrol(fig_han,...
		'style','text',...
		'position', [10 375 60 10],...
		'horizontalalignment', 'left',...
		'string', 'INTEGRATE',...
		'backgroundcolor', math_bkgnd);

	hi_outer = uicontrol(fig_han,...
		'style', 'edit',...
		'pos', [10 345 60 20],...
		'string', '1',...
		'callback', 'iterint(''graphonly'')',...
		'backgroundcolor', math_outer);
		
	lo_outer = uicontrol(fig_han,...
		'style', 'edit',...
		'pos', [10 295 60 20],...
		'string', '0',...
		'callback', 'iterint(''graphonly'')',...
		'backgroundcolor', math_outer);

	hi_inner = uicontrol(fig_han,...
		'style', 'edit',...
		'pos', [80 345 60 20],...
		'string', '(1 - x)^3',...
		'callback', 'iterint(''graphonly'')',...		
		'backgroundcolor', math_inner);

	lo_inner = uicontrol(fig_han,...
		'style', 'edit',...
		'pos', [80 295 60 20],...
		'string', '-x^2',...
		'callback', 'iterint(''graphonly'')',...
		'backgroundcolor', math_inner);

	differential_menu = uicontrol(fig_han,...
		'style', 'popup',...
		'string', '  f(x,y) dx dy|  f(x,y) dy dx',...
		'callback', 'iterint(''graphonly'')',...
		'position', [50 320 140 20],...
		'value', 2);
				
	uicontrol(fig_han,...
		'style','text',...
		'position', [10 265 50 20],...
		'horizontalalignment', 'left',...
		'string', 'WHERE',...
		'backgroundcolor', math_bkgnd);

	uicontrol(fig_han,...
		'style','text',...
		'position', [10 240 50 20],...
		'horizontalalignment', 'right',...
		'string', 'f(x,y) = ',...
		'backgroundcolor', math_bkgnd);
	
	integrand = uicontrol(fig_han,...
		'style', 'edit',...
		'pos', [70 240 120 20],...
		'string', '20*x*y^2 - y^3',...
		'callback', 'iterint(''graphonly'')',...
		'backgroundcolor', math_field);
		
	uicontrol(fig_han,...
		'style', 'frame',...
		'backgroundcolor','k',...
		'position', [10 230 180 1]);

	integral_label = uicontrol(fig_han,...
		'style','text',...
		'position', [10 200 180 20],...
		'backgroundcolor', math_bkgnd);

	uicontrol(fig_han,...
		'style', 'frame',...
		'backgroundcolor', 'k',...
		'position', [10 135 180 1]);

	uicontrol(fig_han,...
		'string', 'Estimate Integral',...
		'position', [10 90 180 35],...
		'callback', 'iterint(''integrate'');');

	uicontrol(fig_han,...
		'string', 'Help',...
		'position', [10 50 180 35],...
		'callback', 'iterint(''help'');');

	uicontrol(fig_han,...
		'string', 'Quit',...
		'position', [10 10 180 35],...
		'callback', 'iterint(''end'');');
		
	axes('position', [.365 .05 .6 .9],...
		'visible','off'); % 360 pixels square

	iterint('graphonly');

	if nargout, result = fig_han; end
	
elseif strcmp(cmd,'graphonly') | strcmp(cmd, 'integrate')

	main_figures = findobj('tag', 'iterint_main');
	if length(main_figures) == 0
		main_figures = iterint('new');
	end

	figure(main_figures(1));

	lo_out = eval( cleanup(get( lo_outer, 'string')));
	hi_out = eval( cleanup(get( hi_outer, 'string')));
	
	if strcmp(cmd, 'graphonly')

		set(integral_label, 'string', []);
		
		cla
		hold on
		
		if get(differential_menu, 'value') == 2 % dy dx	
			x = linspace(lo_out, hi_out, 100);
			
			lo_in = eval(cleanup(vectorize(get(lo_inner, 'string'))));
			if length(lo_in) == 1, lo_in = lo_in + 0*x; end
			hi_in = eval(cleanup(vectorize(get(hi_inner, 'string'))));
			if length(hi_in) == 1, hi_in = hi_in + 0*x; end
			
			for i = 1:100,
				y = linspace( lo_in(i), hi_in(i), 100);
				plot( x(i) + 0*(1:100), y)
			end
		else
			y = linspace(lo_out, hi_out, 100);
			
			lo_in = eval(cleanup(vectorize(get(lo_inner, 'string'))));
			if length(lo_in) == 1, lo_in = lo_in + 0*y; end
			hi_in = eval(cleanup(vectorize(get(hi_inner, 'string'))));
			if length(hi_in) == 1, hi_in = hi_in + 0*y; end
			
			for i = 1:100,
				x = linspace( lo_in(i), hi_in(i), 100);
				plot( x, y(i) + 0*(1:100)),
			end
		end
	
		hold off

	elseif strcmp(cmd, 'integrate')
			
		numerical_iter_int = 0;
	
		if get(differential_menu, 'value') == 2 % dy dx	
			x_vector = linspace(lo_out, hi_out, 150);
			x = x_vector;
			lo_in = eval(cleanup(vectorize( get( lo_inner, 'string'))));
			hi_in = eval(cleanup(vectorize( get( hi_inner, 'string'))));
			if length(lo_in) == 1, lo_in = lo_in + 0*x_vector; end,
			if length(hi_in) == 1, hi_in = hi_in + 0*x_vector; end,
			for i = 1:(length(x_vector) - 1),
				y = linspace( lo_in(i), hi_in(i), 100);
				x = x_vector(i) + 0*(1:length(y));
				z = eval( vectorize( get( integrand, 'string')));
				if length(z) == 1, z = z + 0*y; end,
				delta_x = (hi_out - lo_out)/(length(x_vector) - 1);
				numerical_iter_int = delta_x*trapz(y,z) + numerical_iter_int;
			end,
		else
			y_vector = linspace(lo_out, hi_out, 150);
			y = y_vector;
			lo_in = eval( vectorize( get( lo_inner, 'string')));
			hi_in = eval( vectorize( get( hi_inner, 'string')));
			if length(lo_in) == 1, lo_in = lo_in + 0*y_vector; end,
			if length(hi_in) == 1, hi_in = hi_in + 0*y_vector; end,
			for i = 1:(length(y_vector) - 1),
				x = linspace( lo_in(i), hi_in(i), 100);
				y = y_vector(i) + 0*(1:length(x));
				z = eval( vectorize( get( integrand, 'string')));
				if length(z) == 1, z = z + 0*x; end,
				delta_y = (hi_out - lo_out)/(length(y_vector) - 1);
				numerical_iter_int = delta_y*trapz(x,z) + numerical_iter_int;
			end
		end
		
		set(integral_label, 'string', [ 'Integral  ' num2str( numerical_iter_int)]);
	
	end
	
elseif strcmp(cmd,'help')

	helpwin iterint
	
elseif strcmp(cmd, 'end')

	w = findobj('tag', 'iterint_main');
	for i = 1:length(w)
		delete(w(i))
	end

	% if it's open, return to the main menu

	w = findobj('tag', 'math_main_menu');
	if length(w)
		figure(w(1));
	end

end
