% ODESYS3D
%
% This module is for visualizing solutions to a 3x3 system x'=Ax of 
% linear ordinary differential equations.  Enter the initial conditions 
% x(0), then the 3x3 matrix A, and finally set the range of time values 
% over which to plot.  Click on Update to draw the solution curve.

% REQUIRES: cleanup.m, errmesg.m
%
% EXAMPLE: Type OdeSys3d to run this module.
%          No arguments are needed.
%
% AUTHOR: ?, 1996. 
%		  Revised/Rewritten: William Stein in March, 1998.

function yp = odesys3d(cmd, y)

persistent A 

col=[0.6 0.8 1.0];

if ~nargin
   cmd	=	'new';
   % These are the parameter which will 
   % initially be used. 
   x0  =   [1 -1 2];
   A	=	[-.1 0 0; 0 -.5 1; 0 -1 .5];
   t0	=	0;
   tfinal = 30;
end;

if nargin == 2
   yp=A*y;
   return
end

switch cmd
case 'new',
   
   figs=findobj('tag','tag_odesys3d');
   if length(figs) ~= 0
      figure(figs(1));
      return;
   end
   
   h_fig=figure('Position',[10 10 600 400],...
      'Tag','tag_odesys3d',...
      'NumberTitle','off','Resize','off',...
      'menubar','none', ...
      'numbertitle','off',...
      'Name','3D SYSTEM OF ODEs',...
      'Color', col);
   
   % Create the interface objects:
   %	[Plot] button, x0=([ ],[ ],[ ]), 
   %        ([ ]  [ ]  [ ])
   %    A = ([ ]  [ ]  [ ])
   %		 ([ ]  [ ]  [ ])
   %   [ ] < t < [ ]
   %   [Help]
   %  and the 3d Axes:
   %      along with viewpoint sliders. 
   
   % Create axes
   h_ax=axes('Units','pixels','Position',[265 30 335 335]);   
   
   % [Plot] the solution curve to
   % x'=Ax where
   
   uicontrol(h_fig,'Style','Text',...
      'Position',[50 373 140 15],...
      'String','Plot the solution curve',...
      'HorizontalAlignment','Left',...
      'Background', col);
   uicontrol(h_fig,'Style','Text',...
      'Position',[50 358 140 15],...
      'String','to x''=Ax, where...',...
      'HorizontalAlignment','Left',...
      'Background', col);
   
   % x(0)=([ ] [ ] [ ])
   uicontrol(h_fig,'Style','Text',...
      'Position',[10 330 100 15],...
      'String','x(0) =  (',...
      'HorizontalAlignment','Left',...
      'Background', col);
   for i=1:3
      h_x0(i)=uicontrol(h_fig,'Style','Edit',...
         'Position',[60+50*(i-1) 330 40 20],...
         'String',num2str(x0(i)),...
         'Background','w');
   end
   uicontrol(h_fig,'Style','Text',...
      'Position',[205 330 10 15],...
      'String',')',...
      'HorizontalAlignment','Left',...
      'Background', col); 
   
   % A=[.. 3x3 matrix ..]
   uicontrol(h_fig,'Style','Text',...
      'Position',[20 250 20 15],...
      'String','A =',...
      'HorizontalAlignment','Left',...
      'Background', col);
   uicontrol(h_fig,'Style','frame',...
      'Position',[50 215 1 90]);
   uicontrol(h_fig,'Style','frame',...
      'Position',[50 215 6 1]);
   uicontrol(h_fig,'Style','frame',...
      'Position',[50 305 6 1]);
   uicontrol(h_fig,'Style','frame',...
      'Position',[210 215 1 90]);
   uicontrol(h_fig,'Style','frame',...
      'Position',[205 215 6 1]);
   uicontrol(h_fig,'Style','frame',...
      'Position',[205 305 6 1]);
   for i=1:3
      for j=1:3
         h_A(i,j)=uicontrol(h_fig,'Style','Edit',...
            'Position',[60+50*(j-1) 280-(i-1)*30 40 20],...
            'String',num2str(A(i,j)),...
            'Background', 'w');
      end
   end
   
   
   % and [ ] < t < [ ].
   uicontrol(h_fig,'Style','Text',...
      'String','and t varies from',...
      'Background',col,...
      'HorizontalAlignment','Left',...
      'Position',[10 175 85 15]);
   h_t0=uicontrol(h_fig,'Style','Edit',...
      'Position',[102 175 40 20],...
      'String',num2str(t0),...
      'Background','w');
   uicontrol(h_fig,'Style','Text',...
      'String','to',...
      'Background',col,...
      'Position',[143 175 15 15]);
   h_tfinal=uicontrol(h_fig,'Style','Edit',...
      'Position',[160 175 40 20],...
      'String',num2str(tfinal),...
      'Background','w');
   
   uicontrol(h_fig,'Position',[40 110 110 30],...
      'String','Update',...
      'Callback','OdeSys3d(''Plot'')');
   
   uicontrol(h_fig,'String','Help',...
      'Position',[40 70 110 30],...
      'Callback','OdeSys3d(''Help'')');	
   
   uicontrol(h_fig,'String','Quit',...
      'Position',[40 30 110 30],...
      'Callback','OdeSys3d(''Quit'')');	
   
   
   set(h_fig,'Userdata',...
      [h_x0(1) h_x0(2) h_x0(3) ...
         h_A(1,1) h_A(1,2) h_A(1,3) ...
         h_A(2,1) h_A(2,2) h_A(2,3) ...
         h_A(3,1) h_A(3,2) h_A(3,3) ...
         h_t0 h_tfinal]);
      
      OdeSys3d('Plot')
      
   case 'Plot',
      h_fig=findobj('tag','tag_odesys3d');
      if length(h_fig)==0	% this shouldn't happen,
         OdeSys3d			% but deal with it if it does.	
         return
      end
      h_fig=h_fig(1);
      h=get(h_fig,'Userdata');
      h_x0=h(1:3);
      h_A(1,:)=h(4:6); h_A(2,:)=h(7:9); h_A(3,:)=h(10:12);
      h_t0=h(13); h_tfinal=h(14);
      
      field=0;
      for i=1:3
         s=cleanup(get(h_x0(i),'String'));
         eval([s ';'], ['field=''Entry ' num2str(i) ' of x0'';']);
         if field==0
            x0(i)=str2num(s);
         else
            errmesg(field,'num'); error(lasterr);
         end
      end
      x0=x0';
      
      for i=1:3
         for j=1:3
            s=cleanup(get(h_A(i,j),'String'));
            eval([s ';'], ...
               ['field=''Entry (' num2str(i) ',' num2str(j) ') of A'';']);
            if field==0
               A(i,j)=str2num(s);
            else
               errmesg(field,'num'); error(lasterr);
            end
         end
      end
      
      s_t0=cleanup(get(h_t0,'String'));
      s_tfinal=cleanup(get(h_tfinal,'String'));
      eval([s_t0 ';'], 'field=''Initial value of t'';');
      eval([s_tfinal ';'], 'field=''Final value of t'';');
      if field
         errmesg(field,'num'); error(lasterr);
      end      
      t0=str2num(s_t0);
      tfinal=str2num(s_tfinal);
      
      % compute the solution curve to the ode x'=Ax.		
      [t,y]	=	ode45('odesys3d',[t0,tfinal],x0);
      
      % make the comet plot
      try
	      comet3(y(:,1),y(:,2),y(:,3))
   	   grid on
         hold on
      catch
		end         

      
      % replot since axis settings can't be reset on a comet plot
      theplot = plot3(y(:,1),y(:,2),y(:,3));
      set(theplot,'color','m');
      hold off
      
      % axis labels
      
      xlabel('X');
      ylabel('Y');
      zlabel('Z');
      
      rotate3d on
      
   case 'Help',
      helpwin ('odesys3d') 
      
   case 'Quit',
      delete (findobj ('tag', 'tag_odesys3d')) ;
      w=findobj ('tag', 'math_main_menu') ;
      if length(w)
         figure(w(1));
      end
   end;
   
   
   
   