% POWER SERIES
%
% This program plots things like functions defined by a
% a power series a0 + a1*x + a2*x^2 + a3*x^3 + ...
% 
% INSTRUCTIONS
% 1. Enter the range of x values over which to plot.
% 2. Enter the lower and upper values of n.  For example
%    the lower value of n could be 0 and the upper value 10.
%    This determines how many terms of the series are
%    used in making the plot. 
% 3. Enter the general nth term, for example 'x^n/n'. 
%    In fact this can be any function of x and n whatever,
%    not just a something like ai*x^i.
% 4. If you want to compare the series to a function enter
%    a function in the "FUNCTION" box.  The two will be nicely
%    plotted together for comparison.
%
% WARNING: If you accidentely divide by zero, for example by
% plotting x^n/n with n starting at 0, there will be no error
% message and nothing will be drawn.
%
% William Stein, July, 1998

function PowerSeries (action)

persistent h 

if nargin == 0 
   fig = findobj ('Tag', 'PowerSeries') ;
   if (fig)
      figure (fig(1))
   else
      h = CreatePowerSeriesWindow ;
      PowerSeries ('Update') ;      
   end
   return 
end

switch action
case 'Update',
   hold off      
   
   xmin = eval (cleanup (get(h.xmin, 'String'))) ;
   xmax = eval (cleanup (get(h.xmax, 'String'))) ;
   nmin = eval (cleanup (get(h.nmin, 'String'))) ;
   nmax = eval (cleanup (get(h.nmax, 'String'))) ;
   an = cleanup (get (h.an, 'String')) ;
   func = cleanup (get (h.func, 'String')) ;
   
   f = mysub(an,'n',num2str(nmin)) ;
   for i = nmin+1:nmax
   	f = [f ' + ' mysub(an,'n',num2str(i))] ;
   end
   hold off   
   y=[];
   if ~isempty(func)
   	ezplot (func, xmin, xmax); 
	   han = get(gca,'children') ;
      set(han(1),'color','r')
      hold on 
      y = get(gca,'ylim') ;
   end
   ezplot (f, xmin, xmax) ;
%   if ~isempty(y)
%      set(gca,'ylim',y)
%   end
   
   grid on
   zoom on
   
case 'XSlider',
   set(h.xmin,'string',-floor(get(h.xslider,'value'))) 
   set(h.xmax,'string',floor(get(h.xslider,'value'))) 
   PowerSeries('Update') 
   
case 'NSlider',
   set(h.nmax,'string',floor(get(h.nslider,'value'))) 
   PowerSeries('Update') 
   
case 'Examples',
   e(1).button = '1/(1-x)' ;
   e(1).text = 'The geometric series' ;
   d(1).data = {'-.8','.8','0','5','x^n'} ;
   e(2).button = 'e^x' ;
   e(2).text = 'The exponential function.' ;
   d(2).data = {'-1','3','0','5','x^n/(n!)'} ;
   e(3).button = 'sin(x)' ;
   e(3).text = 'The sine function.' ;
   d(3).data = {'-3*pi','3*pi','0','7','(-1)^n*x^(2*n+1)/((2*n+1)!)'} ;
   e(4).button = 'cos(x)' ;
   e(4).text = 'The cosine function.' ;
   d(4).data = {'-3*pi','3*pi','0','10','(-1)^n*x^(2*n)/((2*n)!)'} ;
   e(5).button = 'atan(x)' ;
   e(5).text = 'The inverse tangent function.' ;
   d(5).data = {'-.8','.8','0','7','(-1)^n*x^(2*n+1)/(2*n+1)'} ;
   i = examples('POWER SERIES', e) ;
   if i
      d=d(i).data;
		set(h.xmin, 'string', d{1}) 
      set(h.xmax, 'string', d{2}) 
		set(h.nmin, 'string', d{3}) 
      set(h.nmax, 'string', d{4}) 
      set(h.an, 'string', d{5}) 
      set(h.func, 'string', e(i).button) 
      PowerSeries Update
   end
      
case 'Help',
   helpwin PowerSeries
   
case 'Quit',
   delete (findobj ('tag', 'PowerSeries')) 
   w = findobj ('tag', 'math_main_menu') ;
   if w
      figure(w(1))
   end
end



function h = CreatePowerSeriesWindow()
col = [.6 .8 1] ;

h0 = figure('Color', col, ...
   'MenuBar','none', ...
   'Name','POWER SERIES', ...
   'NumberTitle','off', ...
   'Position',[15   20   620   410], ...
   'Resize','off', ...
   'Tag','PowerSeries') ;


% axes
h.ax = axes('Parent',h0, ...
   'Units','pixels', ...
   'Position',[40 140 560 240]);

% x range [ ] [ ]
uicontrol(h0,'backgroundcolor',col,...
   'horizontalalignment','left',...
   'style','text',...
   'string','x range: ',...
   'position',[20 75 50 20]) ;
h.xmin = uicontrol('Parent',h0, ...
   'BackgroundColor','w', ...
   'Position',[80 75 40 20], ...
   'String','-3*pi', ...
   'Style','edit', ...
   'Callback','PowerSeries(''Update'');');
h.xmax = uicontrol('Parent',h0, ...
   'BackgroundColor','w', ...
   'Position',[130 75 40 20], ...
   'String','3*pi', ...
   'Style','edit', ...
   'Callback','PowerSeries(''Update'');');
h.xslider = uicontrol(h0,...
   'style','slider',...
   'position', [185 75 145 20], ...
   'min',1,...
   'max',50,...
   'value',2*pi,...
   'callback','PowerSeries(''XSlider'');') ;


% n range [ ] [ ]
uicontrol(h0,'backgroundcolor',col,...
   'style','text',...
   'horizontalalignment','left',...
   'string','n range: ',...
   'position',[20 45 50 20]) ;
h.nmin = uicontrol('Parent',h0, ...
   'BackgroundColor','w', ...
   'Position',[80 45 40 20], ...
   'String','0', ...
   'Style','edit', ...
   'Callback','PowerSeries(''Update'');');
h.nmax = uicontrol('Parent',h0, ...
   'BackgroundColor','w', ...
   'Position',[130 45 40 20], ...
   'String','8', ...
   'Style','edit', ...
   'Callback','PowerSeries(''Update'');');
h.nslider = uicontrol(h0,...
   'style','slider',...
   'position', [185 45 145 20], ...
   'min',1,...
   'max',50,...
   'value',2*pi,...
   'callback','PowerSeries(''NSlider'');') ;


% nth term = [   ]
uicontrol(h0,'backgroundcolor',col,...
   'style','text',...
   'string','nth term: ',...
   'position',[20 15 50 20], ...
   'horizontalalignment', 'left') ;
h.an = uicontrol('Parent',h0, ...
   'BackgroundColor',[.8 .8 1], ...
   'Callback','PowerSeries (''Update'')', ...
   'HorizontalAlignment','left', ...
   'Position',[80 15 250 20], ...
   'String','(-1)^n*x^(2*n+1)/((2*n+1)!)', ...
   'Style','edit');


% FUNCTION TO ADD
uicontrol(h0,'style', 'text',...
   'backgroundcolor',col,...
   'string', 'FUNCTION: ', ...
   'position',[370 90 120 20]) ;
h.func = uicontrol(h0,'style','edit',...
   'backgroundcolor',[1 .8 .8],...
   'horizontalalignment', 'left', ...
   'position',[470 90 120 20], ...
   'callback','powerseries Update') ;

% [Help] [Quit]
uicontrol('Parent',h0, ...
   'Callback','PowerSeries (''Examples'');', ...
   'Position',[480 50 110 30], ...
   'String','Examples');
uicontrol('Parent',h0, ...
   'Callback','PowerSeries (''Help'');', ...
   'Position',[360 15 110 30], ...
   'String','Help');
uicontrol('Parent',h0, ...
   'Callback','PowerSeries (''Quit'');', ...
   'Position',[480 15 110 30], ...
   'String','Quit');



function H = mysub(G, s, t) 
H = '' ;
for i=1:length(G) 
   if (G(i) ~= s)
      H = [H G(i)] ;
   else   
      if i>1 & (('A'<=G(i-1) & G(i-1)<='Z')|...
            ('a'<=G(i-1) & G(i-1)<='z'))
         H = [H G(i)] ;
      elseif i<length(G) & ...
            (('A'<=G(i+1) & G(i+1)<='Z')|...
            ('a'<=G(i+1) & G(i+1)<='z'))
         H = [H G(i)] ;   
      else
         H = [H t] ;
      end
   end
end
