function result = secantlines(cmd)
% Secant Lines
%
% Choose a function f(x) and a domain of x values.  Type these into
% the appropriate boxes.  Pick a value in the domain and type it into
% the x0 box.  Now, click on the curve to select another value x1 and
% view the secant curve through (x0, f(x0)) and (x1, f(x1)).  Drag
% the mouse along the curve to change values.
%
% Information about the points and the slope of the secant line will
% appear below the graph.
%
% 2.1/22-July-1998

% HISTORY:
% 1.0 -- Original by Julie Mitchell (29-Nov-1996).
%        (tanline.m, tlineclk.m, tlinerun.m)
%
% 2.0 -- Rewritten by Tom Insel, Spring 1998,
%        Incorporates features from Deriv.m also by Julie Mitchell
%
% 2.1 -- Updated for MATLAB 5.2
%
% USES: cleanup.m, errmesg.m
%
% NOTES: much like tomepdel, update in parallel

persistent secantlines_line secantlines_point secantlines_extra riserun_text
persistent f_of_x xlow_field xhi_field fig_axes x0_field slope_text x0_text x1_text
% persistent mode_menu


math_bkgnd = [0.6 0.8 1.0];
math_field = [1 1 1];

if ~nargin,

	control_figures = findobj('tag', 'secantlines_main');
	
	if length(control_figures) == 0
		cmd = 'new';
	else
		figure(control_figures(1));
		cmd = 'NOCOMMAND';
	end

end

if strcmp(cmd,'new')
	
	secantlines('makemain')
	secantlines('draw')
	
elseif strcmp(cmd,'makemain')

	fig_han = figure(...
		'name', 'Secant Lines',...
		'Tag', 'secantlines_main',...
		'color', math_bkgnd,...
		'numbertitle', 'off',...
		'position', [10 10 600 400],...
		'resize', 'off',...
		'backingstore', 'off',...
		'WindowButtonUpFcn',...
			'set(gcf,''WindowButtonMotionFcn'','''',''Pointer'',''arrow'');',...
		'WindowButtonDownFcn',...
			'set(gcf,''WindowButtonMotionFcn'',''secantlines(99);'',''Pointer'',''crosshair'');secantlines(99);');

	uicontrol(fig_han,...
		'style', 'frame',...
		'backgroundcolor', math_bkgnd,...
		'position', [10 100 580 1]);

	uicontrol(fig_han,...
		'style', 'text',...
		'horizontalalignment', 'right',...
		'string', 'f(x) =',...
		'backgroundcolor', math_bkgnd,...
		'position', [10 70 45 20]);

	f_of_x = uicontrol(fig_han,...
		'style', 'edit',...
		'backgroundcolor', math_field,...
		'string', 'x^3 - 2*x',...
		'position', [60 70 200 20],...
		'callback', 'secantlines(''draw'');');

	xlow_field = uicontrol(fig_han,...
		'style', 'edit',...
		'backgroundcolor', math_field,...
		'string', '-2',...
		'position', [60 45 50 20],...
		'callback', 'secantlines(''draw'');');

	if strcmp(computer,'MAC2')
		domain_string = ' x ';
	else
		domain_string = '<= x <=';
	end
		
	uicontrol(fig_han,...
		'style', 'text',...
		'string', domain_string,...
		'backgroundcolor', math_bkgnd,...
		'position', [110 45 40 20]);

	xhi_field = uicontrol(fig_han,...
		'style', 'edit',...
		'backgroundcolor', math_field,...
		'string', '2',...
		'position', [150 45 50 20],...
		'callback', 'secantlines(''draw'');');
		
	uicontrol(fig_han,...
		'style', 'text',...
		'horizontalalignment', 'right',...
		'string', 'x0 =',...
		'backgroundcolor', math_bkgnd,...
		'position', [10 20 40 20]);
		
	x0_field = uicontrol(fig_han,...
		'style', 'edit',...
		'backgroundcolor', math_field,...
		'string', '0',...
		'position', [60 20 50 20],...
		'callback', 'secantlines(''draw'');');
		
%	mode_menu = uicontrol(fig_han,...
%		'style', 'popup',...
%		'string', 'rise and run|secant|rectangle',...
%		'position',[140 20 120 20],...
%		'callback', 'secantlines(''draw'');',...
%		'value', 1);
		
	uicontrol(fig_han,...
		'style', 'frame',...
		'backgroundcolor', 'k',...
		'position', [270 10 1 80]);
		
	uicontrol(fig_han,...
		'style', 'frame',...
		'backgroundcolor', 'k',...
		'position', [500 10 1 80]);

	x0_text = uicontrol(fig_han,...
		'style', 'text',...
		'backgroundcolor', math_bkgnd,...
		'position', [272 61 226 20]);

	x1_text = uicontrol(fig_han,...
		'style', 'text',...
		'backgroundcolor', math_bkgnd,...
		'position', [272 44 226 20]);
		
	riserun_text = uicontrol(fig_han,...
		'style', 'text',...
		'backgroundcolor', math_bkgnd,...
		'position', [272 27 226 20]);	
		
	slope_text = uicontrol(fig_han,...
		'style', 'text',...
		'backgroundcolor', math_bkgnd,...
		'position', [272 10 226 20]);

	uicontrol(fig_han,...
		'string', 'Help',...
		'position', [510 55 80 35],...
		'callback', 'helpwin secantlines');

	uicontrol(fig_han,...
		'string', 'Quit',...
		'position', [510 10 80 35],...
		'callback', 'secantlines(''end'');');
		
	fig_axes = axes(...
		'units', 'normalized',...
		'position', [.05 .33 .9 .6],...
		'XLimMode', 'auto',...
		'YLimMode', 'auto');
	
	if nargout, result = fig_han, end
			
elseif strcmp(cmd,'draw') | cmd == 99

	main_figures = findobj('tag', 'secantlines_main');
	if length(main_figures) == 0
		main_figures = secantlines('makemain');
	end
	
	try
		xa = str2num(cleanup(get(xlow_field,'string')));
		xb = str2num(cleanup(get(xhi_field,'string')));
	catch
		errmesg('x domain', 'num'), return
	end

	xlow = min([xa xb]);
	xhi = max([xa xb]);

	try
		x0 = str2num(cleanup(get(x0_field,'string')));
	catch
		errmesg('x domain', 'num'), return
	end

	x0 = max([min([xhi x0]) xlow])
	set(x0_field,'string',num2str(x0))
	
	if cmd == 99
	
		x = x0;
		try
			y0 = eval(vectorize(cleanup(get(f_of_x, 'string'))));
		catch
			errmesg('f(x)', 'fun'),return
		end
	
		p = get(fig_axes, 'CurrentPoint');
		x1  = p(1, 1);
		x = x1;
		try
			y1 = eval(vectorize(cleanup(get(f_of_x, 'string'))));
		catch
			errmesg('f(x)', 'fun'),return
		end
		
		if x1 == x0; x1 = x1 + eps; end  % insurance
		
		slope = (y0 - y1) / (x0 - x1);		

		set(x1_text, 'string', [ '[x1, f(x1)] = [' num2str(x1) ', ' num2str(y1) ']' ])
		
		if strcmp(computer,'MAC2')
			set(riserun_text, 'string', ['f = ', num2str(y1-y0), ' and ', ' x = ', num2str(x1-x0), ', so' ])
			set(slope_text, 'string', [ 'slope = f/x = ' num2str(slope)])
		else
			set(riserun_text, 'string', ['delta f = ', num2str(y1-y0), ' and ', ' delta x = ', num2str(x1-x0), ', so' ])
			set(slope_text, 'string', [ 'slope = delta f/delta x = ' num2str(slope)])
		end
	
%		switch get(mode_menu, 'value')
%			case 1, % rise/run & secant
				set(secantlines_line,...
					'XData', [xlow xhi],...
					'YData', slope*[(xlow-x0) (xhi-x0)] + [y0 y0]);
				if (slope < 0 & x0 < x1) | (slope > 0 & x0 > x1)
					set(secantlines_extra, 'XData', [x0 x0 x1], 'YData', [y0 y1 y1]);
				else
					set(secantlines_extra, 'XData', [x0 x1 x1], 'YData', [y0 y0 y1]);
				end
%			case 2, % secant only
%				set(secantlines_line,...
%					'XData', [xlow xhi],...
%					'YData', slope*[(xlow-x0) (xhi-x0)] + [y0 y0]);
%				set(secantlines_extra, 'XData', x0, 'YData', y0);
%			otherwise, % rectangle only
%				set(secantlines_line, 'XData', [x0 x0 x1 x1 x0], 'YData', [y0 y1 y1 y0 y0]);
%				set(secantlines_extra, 'XData', x0, 'YData', y0);
%		end
				
		set(secantlines_point, 'XData', x1, 'YData', y1);

	else

		x = linspace(xlow,xhi,600);
		try
			plot(x, eval(vectorize(cleanup(get(f_of_x, 'string')))))
		catch
			errmesg('f(x)', 'fun')
		end

		hold on
		
		x = x0; 
		try
			y0 = eval(vectorize(cleanup(get(f_of_x, 'string'))));
		catch
			errmesg('f(x)', 'fun'),return
		end
		
		x1  = (x0 + xhi)/2;
		x = x1;
		try
			y1 = eval(vectorize(cleanup(get(f_of_x, 'string'))));
		catch
			errmesg('f(x)', 'fun'),return
		end
		
		slope = (y0 - y1) / (x0 - x1);		

		plot(x0, y0, 'g*')
		secantlines_line = plot([xlow xhi], slope*[(xlow-x0) (xhi-x0)] + [y0 y0], 'r',...
			'Erase', 'Xor');
		if (slope < 0 & x0 < x1) | (slope > 0 & x0 > x1)
			secantlines_extra = plot([x0 x0 x1], [y0 y1 y1],...
				'Erase', 'Xor', 'Color', [1 .6 0]);
		else
			secantlines_extra = plot([x0 x1 x1], [y0 y0 y1],...
				'Erase', 'Xor', 'Color', [1 .6 0]);
		end
		secantlines_point = plot(x0, y0, 'g*', 'Erase', 'Xor');
		
		hold off
		
		set(x0_text, 'string', [ '[x0, f(x0)] = [' num2str(x0) ', ' num2str(y0) ']' ])
		set([x1_text slope_text riserun_text], 'string', '')
		title('Click & Drag to Select x1')	

	end
	
elseif strcmp(cmd, 'end')

	w = findobj('tag', 'secantlines_main');
	for i = 1:length(w)
		delete(w(i))
	end

	% if it's open, return to the main menu

	w = findobj('tag', 'math_main_menu');
	if length(w)
		figure(w(1));
	end

end;
