function result=snellslaw(cmd,x)
% SNELL'S LAW
%
% Use this module to investigate and minimize travel times through two
% different materials -- typically a person running and swimming or light
% shining through air and water.
%
% Enter the speeds in each material in the appropriate boxes.  Then, enter
% the geometry of the situation: how high to start above the water and how
% deep to finish, as well as the width of the water.
%
% Now, click and drag along the border between air and water to see the
% total travel time between the starting point, the point you choose, and
% the ending point.  You can also see the angles formed at the top and
% the bottom.
%
% 2.1/21-July-1998

%
% HISTORY:
% 1.0 -- Julie Mitchel (2-29-26)
% 2.0 -- revised by tinsel (spring 98).  Modernized, fixed traveltime to be correct
%
% todo -- cleanup, error detect, add angles
% display Snell's law, check certain numbers are positive

math_bkgnd = [0.6 0.8 1.0];
math_field = [1 1 1];
math_water = [0 .5 .3];

persistent hwtext hltext wtext vatext vwtext ray_axes time_axes c_string theline

if ~nargin

	control_figures = findobj('tag', 'refract_main');
	
	if length(control_figures) == 0
		cmd = 'new';
	else
		figure(control_figures(1));
		cmd = 'NOCOMMAND';
	end

end

if strcmp(cmd, 'new')
					
	fig = figure(...
		'name', 'Snell''s Law',...
		'Tag', 'refract_main',...
		'numbertitle', 'off',...
		'position', [10 10 600 400],...
		'resize', 'off',...
		'backingstore', 'off',...
		'color', math_bkgnd,...
		'WindowButtonUpFcn',...
			'set(gcf,''WindowButtonMotionFcn'','''',''Pointer'',''arrow'');',...
		'WindowButtonDownFcn',...
			'set(gcf,''WindowButtonMotionFcn'',''snellslaw(99)'',''Pointer'',''crosshair'');snellslaw(99);');

	uicontrol(fig,...
		'style','frame',...
		'position', [200 10 1 380]);
	
	ray_axes = axes('position', [.4 .1 .55 .8]);
	time_axes = axes('position', [.033 .4 .28 .03], 'YTick', [], 'XTick', [], 'box', 'on');
	
	uicontrol(fig,...
		'style', 'text',...
		'string', 'SPEED...',...
		'horizontalalignment', 'left',...
		'backgroundcolor', math_bkgnd,...
		'position', [10 370 150 20]);
		
	uicontrol(fig,...
		'style', 'text',...
		'string', 'in top medium',...
		'horizontalalignment', 'right',...
		'backgroundcolor', math_bkgnd,...
		'position', [10 350 110 20]);

	vatext = uicontrol(fig,...
		'style', 'edit',...
		'string', '15',...
		'horizontalalignment', 'right',...
		'backgroundcolor', math_field,...
		'position', [130 350 60 20],...
		'callback', 'snellslaw(''repatch'')');

	uicontrol(fig,...
		'style', 'text',...
		'string', 'in bottom medium',...
		'horizontalalignment', 'right',...
		'backgroundcolor', math_bkgnd,...
		'position', [10 325 110 20]);

	vwtext = uicontrol(fig,...
		'style', 'edit',...
		'string','10',...
		'horizontalalignment', 'right',...
		'backgroundcolor', math_field,...
		'position', [130 325 60 20],...
		'callback', 'snellslaw(''repatch'')');

	uicontrol(fig,...
		'style','frame',...
		'position', [10 315 180 1]);
		
	uicontrol(fig,...
		'style', 'text',...
		'string', 'POSITIONS',...
		'horizontalalignment', 'left',...
		'backgroundcolor', math_bkgnd,...
		'position', [10 285 150 20]);
		
	uicontrol(fig,...
		'style', 'text',...
		'string', 'depth of bottom medium',...
		'horizontalalignment', 'right',...
		'backgroundcolor', math_bkgnd,...
		'position', [10 265 130 20]);
		
	hwtext = uicontrol(fig,...
		'style', 'edit',...
		'string', '5',...
		'horizontalalignment', 'right',...
		'backgroundcolor', math_field,...
		'position', [150 265 40 20],...
		'callback', 'snellslaw(''repatch'')');

	uicontrol(fig,...
		'style', 'text',...
		'string', 'height of top medium',...
		'horizontalalignment', 'right',...
		'backgroundcolor', math_bkgnd,...
		'position', [10 240 130 20]);

	hltext = uicontrol(fig,...
		'style', 'edit',...
		'string', '3',...
		'horizontalalignment', 'right',...
		'backgroundcolor', math_field,...
		'position', [150 240 40 20],...
		'callback', 'snellslaw(''repatch'')');

	uicontrol(fig,...
		'style', 'text',...
		'string', 'width',...
		'horizontalalignment', 'right',...
		'backgroundcolor', math_bkgnd,...
		'position', [10 215 130 20]);
		
	wtext = uicontrol(fig,...
		'style', 'edit',...
		'string', '8',...
		'horizontalalignment', 'right',...
		'backgroundcolor', math_field,...
		'position', [150 215 40 20],...
		'callback', 'snellslaw(''repatch'')');
		
	uicontrol(fig,...
		'style', 'frame',...
		'position', [10 205 190 1]);
		
	c_string = uicontrol(fig,...
		'style', 'text',...
		'backgroundcolor', math_bkgnd,...
		'position', [20 100 170 20]);
		
	uicontrol(fig,...
		'string', 'Help',...
		'position', [10 10 85 25],...
		'callback', 'helpwin snellslaw');

	uicontrol(fig,...
		'string', 'Quit',...
		'position', [105 10 85 25],...
		'callback', 'snellslaw(''end'')');
		
	theline = 0;

	snellslaw('repatch');
	snellslaw(99);
					
	if nargout, result = fig, end
	
elseif strcmp(cmd,'repatch') | cmd == 99

	control_figures = findobj('tag', 'refract_main');
	if length(control_figures) == 0
		control_figures = snellslaw('new');
	end
	
	fig = control_figures(1);

	try
		hw=-1*str2num(get(hwtext,'string'));
	    hl=str2num(get(hltext,'string'))-hw;
		width=abs(str2num(get(wtext,'string')));
		va=str2num(get(vatext,'string'));
		vw=str2num(get(vwtext,'string'));
	catch
		errmesg('input data','xxx')
		return
	end

	if strcmp(cmd, 'repatch')

		axes(ray_axes)
		cla
		hold on
		
		theline = 0;
		
		plot([0 width width 0 0],[hw hw 0 0 hw], 'color', math_water)
		
		x = linspace(0, width, 10);
		y = linspace(0, hw, 10);
		for i = 1:10
			plot([x(i) width], [hw y(i)], 'color', math_water);
			plot([0 x(i)], [y(i) 0], 'color', math_water);
		end

		hold off
		
		axis([0 abs(width) hw hl+hw])
		axis('equal')
		set(ray_axes, 'box', 'off')

		x = linspace(0,width);
		times = sqrt((hl+hw)^2 + x.^2)/va +  sqrt((hw)^2 + (width-x).^2)/vw;
		t1 = max(times);
		t0 = min(times);
		td = 0.1 * abs(t1-t0);

		axes(time_axes)
		cla
		axis([t0-td t1+td 0 1])
		
		set(c_string, 'string', '');

	elseif cmd == 99
	
		axes(ray_axes)

		p=get(ray_axes,'CurrentPoint');
		x = min(max(p(1,1),0),width);
		
		theta1 = atan(x / (hl+hw));
		theta2 = atan((width-x)/(-hw));

		if theline
			set(theline, 'xdata', [0 x width], 'ydata', [hl+hw 0 hw])
		else
			hold on
			theline = plot([0 x width], [hl+hw 0 hw], 'erasemode', 'xor');
			hold off
		end

		axes(time_axes)
		traveltime = sqrt((hl+hw)^2 + x^2)/va +  sqrt((hw)^2 + (width-x)^2)/vw;
		
		timedot = findobj(fig, 'tag', 'timedot');

		if length(timedot)
			set(timedot, 'xdata', [traveltime traveltime], 'ydata', [0 1])
		else
			hold on
			plot([traveltime traveltime], [0 1], 'r', 'tag', 'timedot', 'erasemode', 'xor')
			hold off
		end

		set(c_string, 'string', ['bending point C = ', num2str(x)]);

		xlabel(['travel time = ',num2str(traveltime,6)])
%		xlabel(['q1 = ' num2str(theta1) ',  q2 = ' num2str(theta2) ], 'fontname', 'symbol')
		title(['\theta_1 = ' num2str(180*theta1/pi) '^o,  \theta_2 = ' num2str(180*theta2/pi) '^o' ])

	end
	
elseif strcmp(cmd, 'end')

	w = findobj('tag', 'refract_main');
	for i = 1:length(w)
		delete(w(i))
	end

	% if it's open, return to the main menu

	w = findobj('tag', 'math_main_menu');
	if length(w)
		figure(w(1));
	end


end	
