function Substitutor(cmd)
% SUBSTITUTOR
%
% Given an integral Int(f(x)) this program performs a 
% given substitution h(u)=g(x). 
%
% 1. Enter the function to integrate in the first box.  This 
%    function must have ''x'' as its independent variable.
% 2. Enter the substitution in the second box.  This must be an 
%    expression of the form f(x) = g(u).  For example, the substitution 
%    sin(x) = u is valid whereas x = theta/2 is not.




% Author:  Julie Mitchell
% Version: 1.0
% Date:    9-29-96
%
%	This GUI uses Maple's student package to allow students
%	to apply the substitution rule. The input argument, cmd,
%	is used internally and does not have to be specified.
%
%	Example:  Substitutor
%
% REWRITTEN: William Stein, March, 1998.

if ~havesymbolic
   errordlg ('Sorry, this module requires the symbolic toolbox.', ...
      'Subsitutor Error') 
end


global ftext utext fudxtext

h_figs=findobj('Tag','tag_Substitutor');

if h_figs 
   h_fig = h_figs(1);
   if nargin==0
      % When an instance of Substitutor is already running, the
      % command Substitutor just brings it to the foreground.
      figure(h_fig);
      return
   end
end

if ~nargin
   cmd	=	'Initialize';
end;


switch cmd
   case 'Initialize',
   
   % Initialize Maple's student package
   maple('with','student');
   
   % Create new figure
   col = [.6 .8 1] ;
   
   h_fig = figure('Position',[20 40 600 250],...
      'Tag','tag_Substitutor',...
      'NumberTitle','off',...
      'menu', 'none', ...
      'Name','SUBSTITUTOR',...
      'Resize','off',...
      'Color', col);
   
   
   % Int [   ] dx
   uicontrol (h_fig, ...
      'horizontalalignment','left',...
      'position', [60 200 20 20],...
      'style','text',...
      'string','Int',...
      'backgroundcolor',col) ;
   h_ftext = uicontrol(h_fig,... 
      'Position', [80 205 180 20],... 
      'Style','edit',... 
      'string','(1+sin(x))*cos(x)',...
      'Tag','ftext',...
      'BackgroundColor','w', ...
      'Callback','Substitutor(''DoSubstitution'')') ;
   uicontrol (h_fig, ...
      'horizontalalignment','left',...
      'position',[270 200 25 20],...
      'style','text',...
      'string','dx',...
      'backgroundcolor',col) ;
   
   
   
   
   % Int [    ] du
   uicontrol (h_fig, ...
      'horizontalalignment','left',...
      'position',[150 160 40 20],...
      'style','text',...
      'string','=   Int',...
      'backgroundcolor',col) ;
   h_fudxtext = uicontrol(h_fig,... 
      'Position', [189 164 352 18],... 
      'Style','frame',... 
      'BackgroundColor',[.9 .9 .9]); 
   h_fudxtext = uicontrol(h_fig,... 
      'HorizontalAlignment','center',... 
      'Position', [190 165 350 16],... 
      'Style','text',... 
      'string','',...
      'BackgroundColor',[.9 .9 .9]); 
   uicontrol (h_fig, ...
      'horizontalalignment','left',...
      'position',[550 160 25 20],...
      'style','text',...
      'string','du',...
      'backgroundcolor',col) ;
   
   
   
   % Substitute [       ]
   uicontrol (h_fig, ...
      'horizontalalignment','left',...
      'position',[150 95 90 20],...
      'style','text',...
      'string','SUBSTITUTION',...
      'backgroundcolor',col) ;
   h_utext = uicontrol(h_fig,... 
      'HorizontalAlignment','left',... 
      'Position',[250 95 180 20],... 
      'Style','edit',... 
      'string','sin(x) = u',...
      'BackgroundColor','w',...
      'Callback','Substitutor(''DoSubstitution'')') ;
   
   
   uicontrol(h_fig,...
      'Position',[175 15 110 30],...
      'String','Help',...
      'Callback','Substitutor(''Help'');');
   
   uicontrol(h_fig,...
      'Position',[305 15 110 30],...
      'String','Quit',...
      'Callback','Substitutor(''Quit'');');
   
   
   set(h_fig,'Userdata',[h_ftext h_utext h_fudxtext]);
   
   Substitutor('DoSubstitution');
   
case 'Help',
   helpwin Substitutor
   
case 'Quit', 
   delete (findobj ('tag', 'tag_Substitutor')) 
   w = findobj ('tag', 'math_main_menu') ;
   if w
      figure(w(1))
   end
   
case 'DoSubstitution',
   % Retrieve handles and redefine the handle variables.
   h=get(h_fig,'Userdata');
   h_ftext=h(1); h_utext=h(2); h_fudxtext=h(3);
   
   % Get user-entered text
   try
	   x=sym('x'); u=sym('u');
   	f	=	cleanup(get(h_ftext,'string'));
	   field=0; 
   	eval([f ';'], 'field=''f(x)'';');
      
      sub	=	cleanup(get(h_utext,'string'));
   	eval([sub ';',], 'field=''substitution'';') ;
   
	   % create string to send to Maple
   
   	fint_maple	=	['Int(' f ',x)'] ;
   
	   % apply substitution
      sub_eqn	=	maple('changevar',sub,fint_maple,'u');
   catch
      errmesg (field, 'fun') ;
   end
   
   % remove garbage
   good_eqn	=	sub_eqn(5:length(sub_eqn)-3);
   good_eqn = char(simple(sym(good_eqn)));
   
   set(h_fudxtext,'string',good_eqn)
end
