function result = tripleintegral(cmd)
% TRIPLE INTEGRALS
%
% A module for setting up triple integrals and visualizing the region
% of integration.
%
% Use the pop-up menu to choose the order of integration.  Then,
% fill out the six boxes with the limits of integration, making
% sure you use only the appropriate variables in each.  Press
% the GRAPH button to see a picture of the region in three-space
% which these limits describe (as slices perpendicular to the axis
% represented by the outermost variable).
%
% To change the resolution adjust the slider bar. 
%
% Click and drag to rotate the graph.
%
% 2.0/16-July-1998

% HISTORY:
%
% 1.0 -- Alex Gottlieb's original triple integrals module.
% 2.0 -- complete rewrite by Tom Insel (Summer 1998).
%        For MATLAB 5, using "slices" instead of "lines".


persistent a_low a_hi b_low b_hi c_low c_hi order_menu h_slider

math_bkgnd = [0.6 0.8 1.0];
math_field = [1 1 1];

if ~nargin, cmd = 'start'; end

if strcmp(cmd, 'start')

	control_figures = findobj('tag', '3int_control');
	if length(control_figures) == 0
      control_figures = tripleintegral('makecontrol');
      tripleintegral('regraph')
	end

	figure(control_figures(1));
	
elseif strcmp(cmd, 'makecontrol')

	the_fig = figure(...
		'position', [10 10 600 400],...
		'color', math_bkgnd,...
		'name', 'Triple Integrals',...
		'resize', 'off',...
		'tag', '3int_control',...
		'numbertitle', 'off');
		
	uicontrol(the_fig,...
		'position', [10 50 100 20],...
		'backgroundcolor', math_bkgnd,...
		'horizontalalignment', 'left',...
		'style', 'text',...
		'string', 'INTEGRAL');
		
	c_low = uicontrol(the_fig,...
		'position', [30 10 80 20],...
		'style', 'edit',...
		'string', '0',...
		'backgroundcolor', [1 .6 .6]);

	c_hi = uicontrol(the_fig,...
		'position', [30 30 80 20],...
		'style', 'edit',...
		'string', '1',...
		'backgroundcolor', [1 .6 .6]);
		
	b_low = uicontrol(the_fig,...
		'position', [120 10 80 20],...
		'style', 'edit',...
		'string', '0',...
		'backgroundcolor', [.6 1 .6]);

	b_hi = uicontrol(the_fig,...
		'position', [120 30 80 20],...
		'style', 'edit',...
		'string', '1',...
		'backgroundcolor', [.6 1 .6]);
		
	a_low = uicontrol(the_fig,...
		'position', [210 10 80 20],...
		'style', 'edit',...
		'string', '0',...
		'backgroundcolor', [.6 .6 1]);

	a_hi = uicontrol(the_fig,...
		'position', [210 30 80 20],...
		'style', 'edit',...
		'string', '1',...
		'backgroundcolor', [.6 .6 1]);
		
	order_menu = uicontrol(the_fig,...
		'position', [300 18 100 20],...
		'style', 'popupmenu',...
		'string', [' dx dy dz| dx dz dy| dy dx dz|'...
				' dy dz dx| dz dx dy| dz dy dx']);
		
	uicontrol(the_fig,...
		'position', [425 10 80 55],...
		'string', 'Graph',...
		'callback', 'tripleintegral(''regraph'')');
   
   h_slider = uicontrol(the_fig,...
      'position', [510 70 80 16], ...
      'style', 'slider',...
      'min', 10, 'max', 60, ...
      'value', 10);
   
	uicontrol(the_fig,...
		'position', [510 40 80 25],...
		'string', 'Help',...
		'callback', 'helpwin tripleintegral');
		
	uicontrol(the_fig,...
		'position', [510 10 80 25],...
		'string', 'Quit',...
		'callback', 'tripleintegral(''end'')');
		
	axes('units', 'pixels',...
		'position', [10 100 580 290]);
		
	result = the_fig;

elseif strcmp(cmd, 'regraph')

	try
		c0 = str2num(cleanup(get(c_low, 'string')));
	catch
		errmesg('high left limit', 'num')
		return
	end

	try
		c1 = str2num(cleanup(get(c_hi, 'string')));
	catch
		errmesg('low left limit', 'num')
		return
	end
	
   cd = (c1 - c0) / get(h_slider,'value');
   
   b0s = vectorize(cleanup(get(b_low, 'string')));
   b1s = vectorize(cleanup(get(b_hi, 'string')));
   
   a0s = vectorize(cleanup(get(a_low, 'string')));
   a1s = vectorize(cleanup(get(a_hi, 'string')));
   
      
	v = get(order_menu, 'value');
	% 1 = dx dy dz, 2 = dx dz dy, 3 = dy dx dz, 4 = dy dz dx,
	% 5 = dz dx dy, 6 = dz dy dx
	
	if v == 1 | v == 3
		csub = 'z';
	elseif v == 2 | v == 5
		csub = 'y';
	else
		csub = 'x';
	end

	if v == 2 | v == 4
		bsub = 'z';
	elseif v == 1 | v == 6
		bsub = 'y';
	else
		bsub = 'x';
	end
	
	if v == 1 | v == 2
		asub = 'x';
	elseif v == 3 | v == 4
		asub = 'y';
	else
		asub = 'z';
	end

	% Here's the nasty bit.
	%
	% We substitute "c" for the outside variable of integration and "b" for
	% the middle variable.  If we just use strrep(), any basic MATLAB 
	% functions with x, y, or z in their names this could cause trouble.
	% OTOH, the Symbolic Toolbox code here is untested as of 17-July-1998.

%	if havesymbolic
%		SUBS(S,NEW,OLD)
%		b0s = subs(b0s, 'c', csub);
%		b1s = subs(b1s, 'c', csub);
%		a0s = subs(subs(a0s, 'c', csub), 'b', bsub);
%		a1s = subs(subs(a1s, 'c', csub), 'b', bsub);
%	else
		b0s = strrep(b0s, csub, 'c');
		b1s = strrep(b1s, csub, 'c');
		a0s = strrep(strrep(a0s, csub, 'c'), bsub, 'b');
		a1s = strrep(strrep(a1s, csub, 'c'), bsub, 'b');
%	end

	hold on
	cla
	
	for c = c0:cd:c1
		try
			b0 = eval(b0s);
		catch
			errmesg('low middle limit', 'fun'), return
		end
	
		try
			b1 = eval(b1s);
		catch
			errmesg('high middle limit', 'fun'), return
		end
	
		b = linspace(b0, b1, get(h_slider,'value'));
		
		try
			a0 = eval(a0s) + 0*b;
		catch
			errmesg('low right limit', 'fun'), return
		end
	
		try
			a1 = eval(a1s) + 0*b;
		catch
			errmesg('high right limit', 'fun'), return
		end
		
		fill3([a0 fliplr(a1)], [b fliplr(b)], [c+0*b c+0*b], 'r');
	end

	hold off
	
	view(3)
%	axis vis3d
	
	xlabel(asub)
	ylabel(bsub)
	zlabel(csub)
	rotate3d ON

elseif strcmp(cmd, 'end')

	w = findobj('tag', '3int_control');
	for i = 1:length(w)
		delete(w(i))
	end

	% if it's open, return to the main menu

	w = findobj('tag', 'math_main_menu');
	if length(w)
		figure(w(1));
	end

end

function r = havesymbolic()
r = ~isempty(findstr(path,'symbolic'));
